from unittest import main
from test_wisard import WisardTestCase
from test_discriminator import DiscriminatorTestCase
from test_cluswisard import ClusWisardTestCase
from test_kernel_canvas import KernelCanvasTestCase

if __name__ == '__main__':
    main(verbosity=2)



from unittest import TestCase
import wisardpkg as wp
from random import random

class KernelCanvasTestCase(TestCase):

    def test_build(self):
        try:
            dimension = 2
            numberOfKernels = 10
            kc = wp.KernelCanvas(dimension, numberOfKernels, bitsByKernel=4, useDirection=True)
            self.assertIsInstance(kc,wp.KernelCanvas)
        except RuntimeError and TypeError:
            self.fail("kernel canvas instantiation fail!")

    def test_transform(self):
        try:
            kc = wp.KernelCanvas(2, 10, bitsByKernel=4, useDirection=False)

            sequenceData = []
            p = [10*random(),10*random()]
            for i in range(100):
                point = list(p)
                point[0] += i
                point[1] += i
                sequenceData.append(point)

            out = kc.transform(sequenceData)
            for i in range(len(out)):
                self.assertIn(out[i],[0,1])

        except RuntimeError and TypeError:
            self.fail("kernel canvas transform fail!")



from unittest import TestCase
import wisardpkg as wp
import json

class DiscriminatorTestCase(TestCase):

    X = [
        [1,1,1,0,0,0,0,0,0],
        [1,1,1,1,0,0,0,0,0],
        [0,0,0,0,1,1,1,1,1],
        [0,0,0,0,0,1,1,1,1]
    ]

    def test_build(self):
        try:
            d = wp.Discriminator(3,len(self.X[0]))
            self.assertIsInstance(d,wp.Discriminator)
        except TypeError:
            self.fail("discriminator instantiation failed!")

    def test_train(self):
        try:
            d = wp.Discriminator(3,len(self.X[0]))
            d.train(self.X)
        except RuntimeError and TypeError:
            self.fail("train test fail!")

    def test_classify(self):
        try:
            d = wp.Discriminator(3,len(self.X[0]))
            d.train(self.X)
            d.classify(self.X[0])
        except RuntimeError and TypeError:
            self.fail("classify test fail!")

    def test_json(self):
        try:
            d = wp.Discriminator(3,len(self.X[0]))
            d.train(self.X)
            out = d.json()

            import sys
            if sys.version_info[0] < 3:
                self.assertIsInstance(out,unicode)
            else:
                self.assertIsInstance(out,str)

            out2 = json.loads(out)
            self.assertIsInstance(out2,dict)
            d2 = wp.Discriminator(out)
            self.assertSequenceEqual(d.classify(self.X[0]),d2.classify(self.X[0]))
        except RuntimeError and TypeError:
            self.fail("json test fail!")



from unittest import TestCase
import wisardpkg as wp
import json


class ClusWisardTestCase(TestCase):
    X = [
        [1,1,1,0,0,0,0,0,0],
        [1,1,1,1,0,0,0,0,0],
        [1,0,1,0,1,0,1,0,1],
        [0,0,0,0,1,1,1,1,1],
        [0,0,0,0,0,1,1,1,1]
    ]

    y = [
        "cold",
        "cold",
        "cold",
        "hot",
        "hot"
    ]

    y2 = {
        1:"cold",
        3:"hot"
    }

    def test_build(self):
        try:
            addressSize = 3
            minScore = 0.1
            threshold = 10
            discriminatorLimit = 5
            clus = wp.ClusWisard(addressSize,minScore,threshold,discriminatorLimit)
            self.assertIsInstance(clus,wp.ClusWisard)
        except RuntimeError and TypeError:
            self.fail("cluswisard instantiation failed!")

    def test_train_supervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.train(self.X,self.y)
        except RuntimeError and TypeError:
            self.fail("supervised train test fail!")

    def test_train_unsupervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.trainUnsupervised(self.X)
        except RuntimeError and TypeError:
            self.fail("unsupervised train test fail!")

    def test_train_semisupervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.train(self.X,self.y2)
        except RuntimeError and TypeError:
            self.fail("semisupervised train test fail!")

    def test_classify_supervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.train(self.X,self.y)
            out = clus.classify(self.X)
            self.assertSequenceEqual(out,self.y)
        except RuntimeError and TypeError:
            self.fail("supervised classify test fail!")

    def test_classify_unsupervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.trainUnsupervised(self.X)
            out = clus.classifyUnsupervised(self.X)
        except RuntimeError and TypeError:
            self.fail("unsupervised classify test fail!")

    def test_classify_semisupervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.train(self.X,self.y2)
            out = clus.classify(self.X)
            out2 = {k:out[k] for k in self.y2.keys()}
            self.assertSequenceEqual(out2,self.y2)
        except RuntimeError and TypeError:
            self.fail("semisupervised classify test fail!")

    def test_json_supervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.train(self.X,self.y)
            out = clus.json()
            import sys
            if sys.version_info[0] < 3:
                self.assertIsInstance(out,unicode)
            else:
                self.assertIsInstance(out,str)
            out2 = json.loads(out)
            clus2 = wp.ClusWisard(out)
            self.assertSequenceEqual(clus.classify(self.X),clus2.classify(self.X))
        except RuntimeError and TypeError:
            self.fail("supervised json test fail!")

    def test_json_unsupervised(self):
        try:
            clus = wp.ClusWisard(3,0.1,10,5)
            clus.trainUnsupervised(self.X)
            out = clus.json()
            import sys
            if sys.version_info[0] < 3:
                self.assertIsInstance(out,unicode)
            else:
                self.assertIsInstance(out,str)
            out2 = json.loads(out)
            self.assertIsInstance(out2,dict)
            clus2 = wp.ClusWisard(out)
            self.assertSequenceEqual(clus.classifyUnsupervised(self.X),clus2.classifyUnsupervised(self.X))
        except RuntimeError and TypeError:
            self.fail("unsupervised json test fail!")




from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
import sys
import setuptools
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
    f.close()

with open("src/version.h") as f:
    line = f.read()
    __version__ = line.split('"')[1]
    f.close()

__package_name__ = 'symbolicwisardpkg'  # Nome normalizado (minúsculo) para PyPI
__module_name__ = 'symbolicwisardpkg'  # Nome do módulo Python (deve corresponder ao PYBIND11_MODULE)
__src__ = 'src/wisard_bind.cc'

class get_pybind_include(object):
    """Helper class to determine the pybind11 include path
    The purpose of this class is to postpone importing pybind11
    until it is actually installed, so that the ``get_include()``
    method can be invoked. """

    def __init__(self, user=False):
        self.user = user

    def __str__(self):
        import pybind11
        return pybind11.get_include(self.user)


ext_modules = [
    Extension(
        __module_name__,
        [__src__],
        include_dirs=[
            # Path to pybind11 headers
            get_pybind_include(),
            get_pybind_include(user=True)
        ],
        language='c++'
    ),
]


# As of Python 3.6, CCompiler has a `has_flag` method.
# cf http://bugs.python.org/issue26689
def has_flag(compiler, flagname):
    """Return a boolean indicating whether a flag name is supported on
    the specified compiler.
    """
    import tempfile
    with tempfile.NamedTemporaryFile('w', suffix='.cpp') as f:
        f.write('int main (int argc, char **argv) { return 0; }')
        try:
            compiler.compile([f.name], extra_postargs=[flagname])
        except setuptools.distutils.errors.CompileError:
            return False
    return True


def cpp_flag(compiler):
    """Return the -std=c++[11/14] compiler flag.
    The c++14 is prefered over c++11 (when it is available).
    """
    if has_flag(compiler, '-std=c++14'):
        return '-std=c++14'
    elif has_flag(compiler, '-std=c++11'):
        return '-std=c++11'
    else:
        raise RuntimeError('Unsupported compiler -- at least C++11 support '
                           'is needed!')


class BuildExt(build_ext):
    """A custom build extension for adding compiler-specific options."""
    c_opts = {
        'msvc': ['/EHsc'],
        'unix': [],
    }

    if sys.platform == 'darwin':
        c_opts['unix'] += ['-stdlib=libc++', '-mmacosx-version-min=10.7']

    def build_extensions(self):
        ct = self.compiler.compiler_type
        opts = self.c_opts.get(ct, [])
        if ct == 'unix':
            opts.append('-DVERSION_INFO="%s"' % self.distribution.get_version())
            opts.append(cpp_flag(self.compiler))
            if has_flag(self.compiler, '-fvisibility=hidden'):
                opts.append('-fvisibility=hidden')
        elif ct == 'msvc':
            opts.append('/DVERSION_INFO=\\"%s\\"' % self.distribution.get_version())
        for ext in self.extensions:
            ext.extra_compile_args = opts
        build_ext.build_extensions(self)

setup(
    name=__package_name__,
    version=__version__,
    author='Yasmin Forestti Romeiro',
    author_email='y.f.romeiro@gmail.com',
    url='https://github.com/YasminForestti/wisardpkg-symbolic',
    # download_url = 'https://github.com/yasminforestti/wisardpkg/archive/v'+str(__version__)+'.tar.gz',
    description='This project is an extension of the wisardpkg library and provides additional machine learning models based on the WiSARD architecture, including symbolic capabilities. It offers implementations of WiSARD-based models with high performance, easy installation, and a unified usage pattern.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    ext_modules=ext_modules,
    install_requires=['pybind11>=2.2'],
    cmdclass={'build_ext': BuildExt},
    zip_safe=False,
    keywords = ['wisard', 'weithgless', 'neural', 'net', 'symbolic'],
    classifiers=[
        "License :: OSI Approved :: MIT License"
    ],
)



from unittest import TestCase
import wisardpkg as wp
import json

class WisardTestCase(TestCase):
    X = [
        [1,1,1,0,0,0,0,0,0],
        [1,1,1,1,0,0,1,0,0],
        [1,1,1,0,0,0,1,0,0],
        [1,1,1,1,0,0,0,0,0],
        [1,0,1,0,1,0,1,0,1],
        [0,0,0,0,1,1,1,1,1],
        [0,0,1,0,0,1,1,1,1],
        [0,0,1,0,1,1,1,1,1],
        [0,0,0,0,0,1,1,1,1]
    ]

    y = [
        "cold",
        "cold",
        "cold",
        "cold",
        "cold",
        "hot",
        "hot",
        "hot",
        "hot"
    ]

    def test_build(self):
        try:
            wsd = wp.Wisard(3)
            self.assertIsInstance(wsd,wp.Wisard)
        except TypeError:
            self.fail("Wisard instantiation failed!")

    def test_train(self):
        try:
            wsd = wp.Wisard(3)
            wsd.train(self.X,self.y)
        except RuntimeError and TypeError:
            self.fail("train test fail")

    def test_classify(self):
        try:
            wsd = wp.Wisard(3)
            wsd.train(self.X,self.y)
            out = wsd.classify(self.X)
            self.assertSequenceEqual(self.y,out)
        except RuntimeError and TypeError:
            self.fail("classify test fail")

    def test_json(self):
        try:
            wsd = wp.Wisard(3)
            wsd.train(self.X,self.y)
            ojsonout = wsd.json()

            import sys
            if sys.version_info[0] < 3:
                self.assertIsInstance(ojsonout,unicode)
            else:
                self.assertIsInstance(ojsonout,str)

            jsonout = json.loads(ojsonout)
            self.assertIsInstance(jsonout,dict)
            wsd2 = wp.Wisard(ojsonout)
            self.assertSequenceEqual(wsd.classify(self.X),wsd2.classify(self.X))
        except RuntimeError and TypeError:
            self.fail("json test fail")

    def test_sizeof(self):
        try:
            wsd = wp.Wisard(20)
            X = []
            y = []
            from random import randint as ri
            for i in range(300):
                l = [ ri(0,1) if j < 51 else 0 for j in range(100) ]
                X.append(l)
                y.append("cold")

            for i in range(300):
                l = [ ri(0,1) if j > 50 else 0 for j in range(100) ]
                X.append(l)
                y.append("hot")
            wsd.train(X,y)

            self.assertLess(wsd.getsizeof(),len(wsd.json()))
        except RuntimeError and TypeError:
            self.fail("sizeof test fail")

    def test_mental_image(self):
        try:
            wsd = wp.Wisard(3)
            X = [
                [1,1,1,1,1,0,0,0,0],
                [1,1,1,1,0,0,0,0,0],
                [0,0,0,0,1,1,1,1,1],
                [0,0,0,0,0,1,1,1,1]
            ]
            y = ["cold","cold","hot","hot"]
            wsd.train(X,y)
            m = [
                [2,2,2,2,1,0,0,0,0],
                [0,0,0,0,1,2,2,2,2]
            ]
            im = wsd.getMentalImages()
            self.assertSequenceEqual(m[0],im["cold"])
            self.assertSequenceEqual(m[1],im["hot"])
        except RuntimeError and TypeError:
            self.fail("mental image test fail")
